"""
Provides systembridgegui version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgegui` to change this file.

from incremental import Version

__version__ = Version("systembridgegui", 3, 0, 0, dev=15)
__all__ = ["__version__"]
